%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 0. Initial work          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

delete 'feather.avi';
clear all
inputImg=imread('feather.bmp');
inputImg=double(inputImg)/255;

smallLength = size(inputImg,1);
smallWidth = size(inputImg,2);

smallImg1 = inputImg;

% Generate a blank background image
backgImg=zeros(4*smallLength,4*smallWidth,3);
bigLength = size(backgImg,1);
bigWidth = size(backgImg,2);

aviFileName = 'feather';
framesPerSec = '5';

aviobj = avifile(aviFileName,'fps',framesPerSec, 'compression', 'Indeo5'); % initialize .avi file object.

% Add one blank fram at the begining.
frameObj = im2frame(backgImg);
aviobj = addframe(aviobj,frameObj);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 1. Horizontal sliding    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Generate the horizontal slide bar
slideHBar = double(zeros(smallLength, bigWidth, 3));
for i = 1:smallWidth:bigWidth
    slideHBar = superImpose(smallImg1, slideHBar, [1, i]);
end

slideStep = 2;

% slide the small images
widthRem = mod(bigWidth,slideStep);
leftPos = bigWidth - widthRem + 1; % Initial postion of the left sliding postions
rightPos = widthRem - 1;

bigImg = backgImg;
leftRow = 1; % the row sliding to the left
rightRow = 2*smallLength + 1; % the row sliding to the right
bigImg = superImpose(slideHBar, bigImg, [leftRow, leftPos]);
bigImg = superImpose(slideHBar(:,bigWidth-rightPos:bigWidth,:), bigImg, [rightRow, 1]);

for i=1:slideStep:bigWidth-widthRem
    leftPos = leftPos - slideStep;
    bigImg = superImpose(slideHBar, bigImg, [leftRow, leftPos]);
    
    rightPos = rightPos + slideStep;
    bigImg = superImpose(slideHBar(:,bigWidth-rightPos:bigWidth,:), bigImg, [rightRow, 1]);
    
    frameObj = im2frame(bigImg);  
    aviobj = addframe(aviobj,frameObj);
end

%%%%%%%%%%%%%%%%%%%%%%%%%
% 2. Vertical sliding   %
%%%%%%%%%%%%%%%%%%%%%%%%%

% Get vertical bar from the previous image
slideVBar = bigImg(:,smallWidth+1:2*smallWidth,:);
slideStep = 2;

for i=1:slideStep:smallLength
    downPos = i + slideStep;
    if downPos > smallLength + 1
        downPos = smallLength + 1;
    end
    
    bigImg(1:downPos-1,smallWidth+1:2*smallWidth,:)=0;
    bigImg = superImpose(slideVBar, bigImg, [downPos, smallWidth+1]);
    bigImg(1:downPos-1,3*smallWidth+1:4*smallWidth,:)=0;
    bigImg = superImpose(slideVBar, bigImg, [downPos, 3*smallWidth+1]);

    frameObj = im2frame(bigImg);  
    aviobj = addframe(aviobj,frameObj);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 3. Small image spiral    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%

spiralStep = 6;
resizeStep = 2;
% resizeStep = ceil(smallWidth/(360/spiralStep));

% image map of the previouse one
A = [1 0 1 0; 0 1 0 1; 1 0 1 0; 0 1 0 1];
[spiralRow, spiralCol] = find(A);

bigImg = backgImg;

rowCenter = floor(smallLength/2);
colCenter = floor(smallWidth/2);
angle = 0;
for i = smallWidth:-resizeStep:1
    angle = angle + spiralStep;
    
    X = imrotate(smallImg1, angle, 'bilinear', 'crop');
    
    X1 = imresize(X, [i, i]);

    for j = 1:size(spiralRow,1)
        bigImg = superImpose_center(X1, bigImg, [rowCenter + (spiralRow(j)-1)*smallLength, colCenter + (spiralCol(j)-1)*smallWidth]); 
    end
    
    frameObj = im2frame(bigImg);  
    aviobj = addframe(aviobj,frameObj);
end

frameObj = im2frame(backgImg);
aviobj = addframe(aviobj,frameObj);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 4. Sliding from corner    %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% make an up-right corner image
cornerImg = double(zeros(2*smallLength,2*smallWidth,3));
A = [1 1; 1 1];
[cornerRow, cornerCol] = find(A);

for i=1:size(cornerRow,1)
    cornerImg = superImpose(smallImg1, cornerImg, [(cornerRow(i)-1)*smallLength+1, (cornerCol(i)-1)*smallWidth+1]); 
end

bigImg = backgImg;
slideStep = 2;
slideRem = mod(2*smallWidth,slideStep);
% compute the initial position of the up right corner
slidePos = slideRem; 

for i=1:slideStep:2*smallWidth-slideRem
    slidePos = slidePos + slideStep;
    bigImg = superImpose(cornerImg(size(cornerImg,1)-slidePos+1:size(cornerImg,1), 1:slidePos, :), bigImg, [1,bigWidth-slidePos+1]);
    
    % Get the image for other 3 corner by symetry of the coordinates
    bigImg = getFoldImg(bigImg, 'r2l');
    bigImg = getFoldImg(bigImg, 'u2d');
    frameObj = im2frame(bigImg);  
    aviobj = addframe(aviobj,frameObj);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% 5. Spiral and eliminate the big Iamge  %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

rotateStep = 0.5;
elimStep = 2;
angle = 0;
R1 = sqrt(bigWidth^2 + bigLength^2); % squre of the radius

for i = 0:elimStep:R1
    angle = angle + rotateStep;
    bigImg = imrotate(bigImg, angle, 'bilinear', 'crop');
    [elimRows, elimCols]=find(bigImg(:,:,1)+bigImg(:,:,2)+bigImg(:,:,3)~=0);
    
    if size(elimRows,1) == 0
        break
    end
    
    for j = 1:size(elimRows,1)
        d = sqrt((bigLength/2-elimRows(j))^2+(bigWidth/2-elimCols(j))^2);
        if d < i
            bigImg(elimRows(j),elimCols(j),:) = 0;
        end
    end
    frameObj = im2frame(bigImg);  
    aviobj = addframe(aviobj,frameObj);
end

aviobj = close(aviobj);
% movie(frame,1);

disp 'done!'

