%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% This function is used to super impose an small image on a big image according to the up-left corner 
% position of the smallImage. 
% Inputs: 
%   smallImg - matrix of the small image
%   bigImg - matrix of the big image
%   position - the vector of up-left corner position
% Outputs:
%   outputImag - the matrix of the result matrix
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [outputImg] = superImpose (smallImg, bigImg, position)

    smallLength = size(smallImg,1);
    smallWidth = size(smallImg,2);
    bigLength = size(bigImg,1);
    bigWidth = size(bigImg,2);
    
    if (position(1) >= bigLength) | (position(2) >= bigWidth)
        outputImg = bigImg;
        return
    end

    bigRowBottom = position(1) + smallLength - 1;
    if bigRowBottom > bigLength
        bigRowBottom = bigLength;
    end
    smallRowBottom = bigRowBottom - position(1) + 1;
    
    bigColRight = position(2) + smallWidth - 1;
    if bigColRight > bigWidth
        bigColRight = bigWidth;
    end
    smallColRight = bigColRight - position(2) + 1;
    
    bigImg(position(1):bigRowBottom, position(2):bigColRight, :)=smallImg(1:smallRowBottom, 1:smallColRight, :);
    outputImg = bigImg;
